<?php

require_once 'Git.php';

class CF_GitRepo extends GitRepo
{

    protected static $ssh_agent_bin = '/usr/bin/ssh-agent';
    protected static $ssh_add_bin = '/usr/bin/ssh-add';
    protected $username = "";

    function __construct($repo_path = null, $create_new = false, $_init = true, $username = '')
    {
        $this->username = $username;
        parent::__construct($repo_path, $create_new, $_init);
    }

    public function configure($author, $email)
    {
        $this->run(sprintf("config user.email %s",escapeshellarg($email)));
        $this->run(sprintf("config user.name %s",escapeshellarg($author)));
        return true;
    }

    /**
     * Runs git command through wrapper script for remote connection
     * @param  [type] $command [description]
     * @param  [type] $key     [description]
     * @return [type]          [description]
     */
    function _get_remote_command($command,$pkey)
    {
        log_message('debug','Git wrapper command: '.$command);
        $scriptPath = get_dc_update_script_template_path();
        $arg1 = escapeshellarg($scriptPath);
        $arg2 = escapeshellarg($pkey);
        $arg3 = escapeshellarg($command);
        $script = get_dc_update_script_template_path().DIRECTORY_SEPARATOR.'git-wrapper.sh';
        $execCommand = sprintf("%s %s %s %s",$script,$arg1,$arg2,$arg3);
        return $execCommand;
    }


    /**
     * Run a command in the git repository
     *
     * Accepts a shell command to run
     *
     * @access  protected
     * @param   string  command to run
     * @return  string
     */
    public function run_command($command,$pkey = false)
    {
        log_message('debug','Git command: '.$command);
        $descriptorspec = array(
            1 => array('pipe', 'w'),
            2 => array('pipe', 'w'),
        );
        $pipes = array();
        $apacheHome = get_dc_apache_home(); // for new version of git
        if (empty($apacheHome))
        {
            $apacheHome = '/tmp';
        }

        if (!empty($pkey))
        {
            $command = $this->_get_remote_command($command,$pkey);
        }

        $resource = proc_open($command, $descriptorspec, $pipes, $this->repo_path,array('HOME'=>$apacheHome));

        $stdout = stream_get_contents($pipes[1]);
        $stderr = stream_get_contents($pipes[2]);
        foreach ($pipes as $pipe)
        {
            fclose($pipe);
        }
        log_message('debug','Git stdout: '.$stdout);
        log_message('debug','Git stderr: '.$stderr);
        $status = trim(proc_close($resource));
        if ($status && !empty($stderr))
            throw new Exception($stderr);

        return !empty($stderr) ? $stderr : $stdout;
    }

    public static function get_ssh_agent_bin()
    {
        return self::$ssh_agent_bin;
    }

    public static function get_ssh_add_bin()
    {
        return self::$ssh_add_bin;
    }

    public static function get_key_path()
    {
        return self::$key_path;
    }

    public function get_user_name()
    {
        return $this->username;
    }

    /**
     * Run a git command in the git repository
     *
     * Accepts a git command to run
     *
     * @access  public
     * @param   string  command to run
     * @return  string
     */
    public function run($command,$pkey=false)
    {
        if (empty($pkey))
        {
            $command = Git::get_bin()." ".$command;
        }

        return $this->run_command($command,$pkey);
    }

    public function commit($message = "",$author="",$authorEmail="",$userEmail="")
    {
        $this->configure($author,$authorEmail);
        $command = sprintf('diff --quiet --exit-code --cached || %s commit -av -m %s --author=""%s" <%s>"', Git::get_bin(), escapeshellarg($message),escapeshellarg($this->username),escapeshellarg($userEmail));

        return $this->run($command);
    }

    /**
     * Push specific branch to a remote
     *
     * Accepts the name of the remote and local branch
     *
     * @param string $remote
     * @param string $branch
     * @return string
     */
    public function push($remote='origin', $branch="master")
    {
        return $this->run("push $remote $branch:$branch");
    }

    public function checkout_remote_branch($branch)
    {
        return $this->run("checkout -B $branch origin/$branch");
    }

    function reset_hard($branch='master')
    {

        return $this->run("reset --hard origin/$branch");
    }

    function pre_fetch()
    {
        log_message('debug',"Entering pre-fetch script step");
        $prefetchScript = get_dc_update_script_template_path().DIRECTORY_SEPARATOR.'pre-fetch.sh';
        $prefetchScriptParameter = get_dc_update_script_path().DIRECTORY_SEPARATOR.'params.sh';

        log_message('debug',"Pre-fetch script path is :: ".$prefetchScript);
        if(file_exists($prefetchScriptParameter))
        {
             log_message('debug',"Parameter file detected at $prefetchScriptParameter. Running prefetch script from the location :: ".$prefetchScript);
             return $this->run_command($prefetchScript);
        }
        log_message('debug',"Pre-fetch script skipped");
    }

    function post_push()
    {
        log_message('debug',"Entering post_push script step");
        $postUpdateScript = get_dc_update_script_template_path().DIRECTORY_SEPARATOR.'post-update.sh';
        $prefetchScriptParameter = get_dc_update_script_path().DIRECTORY_SEPARATOR.'params.sh';

        log_message('debug',"post-update script path is :: ".$postUpdateScript);
        if(file_exists($prefetchScriptParameter))
        {
             log_message('debug',"Parameter file detected at $prefetchScriptParameter. Running post-update script from the location :: ".$postUpdateScript);
             return $this->run_command($postUpdateScript);
        }
        log_message('debug',"post update script skipped");
    }

    function clone_remote_mirror($source,$branch=null)
    {
        $pKey = get_dc_user_ssh_key();
        $localWorkingBranch = get_dc_git_working_branch();
        $command = sprintf('clone %s %s -b %s',$source,$this->repo_path,$branch);
        $this->run($command,$pKey);
        $renameCommand = sprintf("branch -m %s",$localWorkingBranch);
        $this->run($renameCommand);
    }

}

?>