<?php

namespace models;

use Interop\Container\ContainerInterface;

/**
 * Class SettingsModel
 * @package models
 */
class SettingsModel implements ISettings
{
    const REQUIRED_RULE_NAME = 'required';
    /**
     * @var array
     */
    private $allowedKeys = [
        'domain_controller' => 'string|' . self::REQUIRED_RULE_NAME,
        'base_dn' => 'string|' . self::REQUIRED_RULE_NAME,
        'login_attribute' => 'string|' . self::REQUIRED_RULE_NAME,
        'port' => 'int',
        'use_ssl' => 'bool',
        'use_tls' => 'bool',
        'timeout' => 'int',
        'ldap_filter' => 'string',
        'admin_username' => 'string|' . self::REQUIRED_RULE_NAME,
        'admin_password' => 'string'
    ];

    /**
     * @var ContainerInterface
     */
    private $container;

    /**
     * SettingsModel constructor.
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }


    /**
     * @param array $data
     *
     * @return mixed|void
     * @throws \Exception
     */
    public function validate(array $data)
    {
        $errors = [];
        $this->validateExistingData($data, $errors);
        $this->validateRequiredFields($data, $errors);

        if(sizeof($errors) > 0){
            throw new \Exception(implode('  ' , $errors));
        }

        return true;
    }

    /**
     * @param $data
     * @param $errors
     */
    private function validateExistingData($data, &$errors){
        foreach ($data as $key => $value) {
            if (key_exists($key, $this->allowedKeys)) {
                foreach (explode('|', $this->allowedKeys[$key]) as $rule) {
                    $key = ucfirst($key);
                    switch ($rule) {
                        case 'int':
                            if (!is_numeric($value)) {
                                $errors[] = $key . ' must be integer.';
                            }
                            break;
                        case 'bool':
                            if ($value != 'true' && $value != 'false') {
                                $errors[] = $key . ' must be `true` or `false`.';
                            }
                            break;
                        case 'required':
                            if ($value == '') {
                                $errors[] = $key . ' field is required.';
                            }
                            break;
                    }
                }
            }
        }
    }

    /**
     * @param $data
     * @param $errors
     */
    private function validateRequiredFields($data, &$errors){
        foreach ($this->allowedKeys as $key => $value) {
            $rules = explode('|', $value);
            if(in_array(self::REQUIRED_RULE_NAME, $rules) && (empty($data[$key]) || !isset($data[$key]))){
                $errors[] = ucfirst($key) . ' field is required.';
            }
        }
    }

    /**
     * @return array|mixed
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function getAll()
    {
        $settings = $this->container->get('ldap');
        $settings['domain_controllers'] = reset($settings['domain_controllers']);
        return $settings;
    }


    /**
     * @return array|mixed
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function getSettingsWithoutPassword()
    {
        $settings = $this->getAll();
        $settings['admin_password'] = $settings['admin_password'] == '' ? 'Password is not set' : 'Password is set';
        $settings['domain_controller'] = $settings['domain_controllers'];
        unset($settings['domain_controllers']);
        return $settings;
    }

    /**
     * @param array $data
     * @return bool
     */
    public function update(array $data)
    {
        $logger = $this->container->logger;
        $logger->debug("SettingsModel::update()", $data);

        $dataToUpdate = $this->prepareData($data);
        $logger->debug("SettingsModel::update(), dataToUpdate=", $dataToUpdate);

        return $this->writeConfig($dataToUpdate);
    }

    /**
     * @param $data
     * @return array
     */
    private function prepareData($data)
    {
        $oldSettings = $this->getAll();
        foreach ($oldSettings as $key => $value) {
            if (is_bool($value)) {
                $oldSettings[$key] = $value ? 'true' : 'false';
            }
        }
        if (isset($data['domain_controller'])) {
            $data['domain_controllers'] = $data['domain_controller'];
            unset($data['domain_controller']);
        }
        return array_merge($oldSettings, $data);
    }

    /**
     * @param $newSettings
     * @return bool
     */
    private function writeConfig($newSettings)
    {
        $settings = file_get_contents(CONFIG_PATH . '/settings.ldap.template');

        $newSettingsContent = vsprintf($settings, $newSettings);
        $this->container->logger->debug("SettingsModel::writeConfig()", array('$newSettingsContent', $newSettingsContent) );

        return (boolean) file_put_contents(CONFIG_PATH . '/settings.ldap.php', $newSettingsContent);
    }

    /**
     * @return bool
     */
    public function makeBackup()
    {
        return (boolean)copy(CONFIG_PATH . '/settings.ldap.php',
            CONFIG_PATH . '/config_dump/settings.ldap.php.' . date('Y-m-d_H:i:s'));
    }
}
